
function ChamferCurve()
{
	this.addInput("Curves", "objectlist");
	this.addInput("Dist1", "numarray");
	this.addInput("Dist2", "numarray");
	this.addInput("Pattern","numarray");
	this.addOutput("Chamfer", "objectlist");
	this.properties = {Dist1: [1], Dist2: [1], Select:["None","None","Sharp Points", "Done", "Reset"], Memory:""};
	this.cornerPoints = moi.geometryDatabase.createObjectList();
	this.stopExecution = false;
}

ChamferCurve.title = "ChamferCurve";
ChamferCurve.desc = "Chamfer Curve node * Use the select dropdown menu to pick corner points";

ChamferCurve.prototype.onPropertyChange = function ()
{
	// get select property mode
	var selectionMode = this.properties.Select[0];
	// if select mode == "none" : other properties have changed
	if(selectionMode == "None"){if(this.cornerPoints!=null)moi.geometryDatabase.removeObjects( this.cornerPoints );} // remove corner points from the scene
	// if select mode == "sharp points" : user wants to select corner points.
	// get all curve objects
	var inputs = this.getInputData(0, moi.geometryDatabase.createObjectList());
	
	if(selectionMode == "Sharp Points" || selectionMode == "Done" || selectionMode == "Reset")
	{
		// get all segments start points
		var allStartPoints = new pointArray();
		for(inpI=0; inpI<inputs.length; inpI++)
		{
			if(inputs.item(inpI).type == 2) // only work on curves
			{
				var segments = inputs.item(inpI).getSubObjects();
				for(segI=0; segI<segments.length; segI++)
				{
					allStartPoints.pushPoint(segments.item(segI).getStartPt());
				}
				// get corner points to display in the scene
				if(selectionMode == "Sharp Points")
				{
					var filletFacToGetCorners = moi.command.createFactory('fillet');
					var tempInput = moi.geometryDatabase.createObjectList();
					tempInput.addObject(inputs.item(inpI));
					filletFacToGetCorners.setInput(0, tempInput);
					filletFacToGetCorners.generateVertices();
					var cp = filletFacToGetCorners.getCreatedObjects();
					for(CPI=0; CPI<cp.length; CPI++)
					{
						this.cornerPoints.addObject(cp.item(CPI));
					}
					filletFacToGetCorners.cancel();
				}
				this.stopExecution = false;
			}else{this.stopExecution = true; break;} // otherwise stop everything!
		}
	}
	// if everything is ok
	if(this.getInputData(0, null, true)!=null && !this.stopExecution)
	{
		// add an object picker
		var objectPicker = moi.ui.createObjectPicker();
		// "sharp points": 
		if(selectionMode == "Sharp Points")
		{
			// add corner point objects to the scene
			moi.geometryDatabase.addObjects(this.cornerPoints);
			// constraint selection filter to points
			objectPicker.allowPointObjects();
			objectPicker.objects.unlockSelection();
			moi.selection.clearSelectionFilters();
			moi.selection.setFilter( 'Types', 'Points', true );
			// wait for user to choose the "Done"
		}
		// "Done":
		if(selectionMode == "Done")
		{
			// close the object picker and clear it's constraints
			objectPicker.done();
			objectPicker.resetAllow();
			moi.selection.clearSelectionFilters();
			moi.geometryDatabase.deselectAll();
			// remove corner points from the scene
			if(this.cornerPoints!=null)moi.geometryDatabase.removeObjects( this.cornerPoints );
			// clear the corner points objectlist
			var CPlength = this.cornerPoints.length;
			for(CPRI=CPlength; CPRI>0; CPRI--)
			{
				this.cornerPoints.removeObjectAt(CPRI-1);
			}
			// get selected point objects
			var selectedPoints = objectPicker.objects;
			// console.log(selectedPoints.item(0).pt.x);
			var temporaryBooleanList = moi.createList();
			// compare all segments start points with selected points to make a list of 0/1 values
			for(startPointsI=0; startPointsI<allStartPoints.getLength(); startPointsI++)
			{
				if(IsSamePoint(allStartPoints.getPoint(startPointsI), selectedPoints))
				{
					temporaryBooleanList.add( true );
				}else
				{
					temporaryBooleanList.add( false );
				}
			}
			// set memory
			this.properties.Memory = replacer(temporaryBooleanList).toString();
			// console.log(this.properties.Memory);
			this.properties.Select[0] = "None";
			// refresh node info!
			this.graph.refreshNodeInfo(this.id);
			// let the "onExecute" do the rest!
		}
		// "Reset"
		if(selectionMode == "Reset")
		{
			// remove corner points from the scene
			if(this.cornerPoints!=null)moi.geometryDatabase.removeObjects( this.cornerPoints );
			// clear memory
			this.properties.Memory = "";
			this.properties.Select[0] = "None";
		}
	}else
	{
		if(selectionMode == "Sharp Points" || selectionMode == "Done" || selectionMode == "Reset")
		{
			this.properties.Select[0] = "None";
			this.graph.refreshNodeInfo(this.id);
		}
	}
}

ChamferCurve.prototype.onExecute = function ()
{
	try // extra safety!
	{
		var that = this;
		var outputObjects = moi.geometryDatabase.createObjectList();
		// sync input and property
		this.properties.Dist1 = this.getInputData(1, this.properties.Dist1);
		var Distance1 = this.properties.Dist1[0];
		this.properties.Dist2 = this.getInputData(2, this.properties.Dist2);
		var Distance2 = this.properties.Dist2[0];
		if(this.getInputData(2, null, true) == null)
		{
			this.properties.Dist2 = this.getInputData(1, this.properties.Dist1);
			var Distance2 = this.properties.Dist1[0];
		}
		// get all curve objects
		var inputs = this.getInputData(0, moi.geometryDatabase.createObjectList());
		// get all segments start points
		var startPoints = [];
		for(inpI=0; inpI<inputs.length; inpI++)
		{
			if(inputs.item(inpI).type == 2)
			{
				var segments = inputs.item(inpI).getSubObjects();
				for(segI=0; segI<segments.length; segI++)
				{
					startPoints.push(segments.item(segI).getStartPt());
				}
				this.stopExecution = false;
			}else{this.stopExecution = true; break;}
		}
		// check the sharp points pattern input
		if(this.getInputData(3, null, true)!=null)
		{
			var inpPat = this.getInputData(3, null);
			this.properties.Memory = inpPat;
		}
		if(this.getInputData(0, null, true)!=null && !this.stopExecution)
		{
			// check if memory is empty or if it doesn't match segment length
			var memory = this.properties.Memory;
			// if so : call setDefaultSegmentList
			if(memory == "" || this.properties.Memory.toString().replace(/,/g,'').length !== startPoints.length)
			{
				setDefaultSegmentList(inputs, that)
			}
			// do the filleting precedure based on memory like this:
			// get memory as a string of 01
			memory = this.properties.Memory.toString().replace(/,/g,'');
			// memory is a list of 0/1 values so it needs to convert to true/false values using replacer function
			var startPointsList = replacer(memory);
			// go for chamfer!
			var counter = 0;
			for(inpII=0; inpII<inputs.length; inpII++)
			{
				// make a temporary object list to keep every curve on it's own chamfer factory
				// chamfer factory acts differently when you set multiple curves as input
				var temporaryCurve = moi.geometryDatabase.createObjectList();
				temporaryCurve.addObject(inputs.item(inpII));
				// get it's segments length + counter to manage memory
				segmentsLength = temporaryCurve.item(0).getSubObjects().length+counter;
				// do a for loop with segmentsLength value to make a temp list
				var temporaryList = moi.createList();
				for(segII=counter; segII<segmentsLength; segII++)
				{
					temporaryList.add(startPointsList.item(segII));
					counter++;
				}
				var filletFactory = moi.command.createFactory('chamfer');
				filletFactory.setInput(0, temporaryCurve);
				// filletFactory.setInput(1, false); // It works only on edges
				filletFactory.setInput(2, temporaryList);
				filletFactory.setInput(3, Distance1);
				filletFactory.setInput(4, Distance2);
				filletFactory.update();
				var out = filletFactory.getCreatedObjects();
				
				outputObjects.addObject(out.item(0));
				filletFactory.cancel();
			}
			this.setOutputData(0, outputObjects);
			this.inputs[3].label="Pattern: "+startPoints.length;
			this.graph.refreshNodeInfo(this.id);
		}
		// update box color
		if(this.inputs[0].link !== null) { if (outputObjects.length === 0) {this.boxcolor = "#F80"} else {this.boxcolor = "#0F5"}} else {this.boxcolor = "#F05"}
	}catch(e){/* console.log(e) */}
	moi.geometryDatabase.getObjects().unlockSelection();
}

LiteGraph.registerNodeType("Construct/ChamferCurve", ChamferCurve);

// setDefaultSegmentList function
// make a list of 0/1 values for curve objectlist
function setDefaultSegmentList(curve, that)
{
	var tempCurve = curve;
	var tempStartPoints = [];
	var tempMemory = [];
	var cornerPoints = moi.geometryDatabase.createObjectlist();
	for(tempI=0; tempI<tempCurve.length; tempI++)
	{
		var tempSegments = tempCurve.item(tempI).getSubObjects();
		for(temSI=0; temSI<tempSegments.length; temSI++)
		{
			tempStartPoints.push(tempSegments.item(temSI).getStartPt());
		}
		var filletFacToGetCorners = moi.command.createFactory('fillet');
		var tempInput = moi.geometryDatabase.createObjectList();
		tempInput.addObject(tempCurve.item(tempI));
		filletFacToGetCorners.setInput(0, tempInput);
		filletFacToGetCorners.generateVertices();
		var cp = filletFacToGetCorners.getCreatedObjects();
		for(CPI=0; CPI<cp.length; CPI++)
		{
			cornerPoints.addObject(cp.item(CPI));
		}
		filletFacToGetCorners.cancel();
	}
	for(TSPI=0; TSPI<tempStartPoints.length; TSPI++)
	{
		if(IsSamePoint(tempStartPoints[TSPI],cornerPoints))
		tempMemory.push(1);
		else
		tempMemory.push(0);
	}
	that.properties.Memory = tempMemory;
}

// replacer function
// convert a 0/1 array to true/false list and vice versa
function replacer(array)
{
	var value = array[0];
	var arrayLength = array.length;
	if (typeof value === 'boolean')
	{
		var replacedArray = [];
		for(z=0;z<arrayLength;z++)
		{
			if(array[z]){replacedArray.push("1");}else{replacedArray.push("0");}
		}
	}
	if (typeof value === 'string')
	{
		var replacedArray = moi.createList();
		for(z=0;z<arrayLength;z++)
		{
			if(array[z]=="1"){replacedArray.add(true);}else{replacedArray.add(false);}
		}
	}
	return replacedArray;
}

// IsSamePoint function
// compare a point with a point objectlist
// used for filtering the G0 points out of segments start points
function IsSamePoint(pt1,pt2)
{
	var isSamePoint = false;
	for(kkj=0; kkj<pt2.length; kkj++)
	{
		// console.log(pt1.x +"\t"+ pt2.item(kkj).pt.x +"\t"+ pt1.y +"\t"+ pt2.item(kkj).pt.y +"\t"+ pt1.z +"\t"+ pt2.item(kkj).pt.z);
		if(pt1.x == pt2.item(kkj).pt.x && pt1.y == pt2.item(kkj).pt.y && pt1.z == pt2.item(kkj).pt.z)
		{
			isSamePoint = true;
			break;
		}else{isSamePoint = false;}
	}
	return isSamePoint;
}